@@ -15,7 +15,7 @@ use Carp;
 use List::Util qw(uniq);
 use POSIX qw(strftime);
 
-our $VERSION = '0.059'; # TRIAL VERSION
+our $VERSION = '0.060'; # TRIAL VERSION
 
 use Sub::Quote;
 
@@ -105,9 +105,10 @@ has 'bag_encoding' => (
 );
 
 has 'bag_info' => (
-    is      => 'rw',
-    lazy    => 1,
-    builder => '_build_bag_info',
+    is        => 'rw',
+    lazy      => 1,
+    builder   => '_build_bag_info',
+    predicate => 1
 );
 
 has 'errors' => (
@@ -319,6 +320,7 @@ sub __build_xxxmanifest_entries {
   foreach my $xxmanifest_file (@xxmanifests) {
     die("Cannot open $xxmanifest_file: $!") unless (open(my $XXMANIFEST,"<:encoding(utf8)", $xxmanifest_file));
     my $algo = $xxmanifest_file;
+    $algo =~ s#\\#/#g; # change Backslash to slash to prevent next regex for clashes
     $algo =~ s#//#/#g; # to fix problems with double path-separators
     $algo =~ s#^($bag_path/).bagit/#$1#; # FIXME: only for dotbagit-variant, if dotbagit will be outdated, this should be removed
     $algo =~ s#^$bag_path/##;
@@ -373,6 +375,7 @@ sub _build_payload_files{
     }
     #print "name: ".$File::Find::name."\n";
   }, $payload_dir);
+  @payload = sort @ payload;
   return wantarray ? @payload : \@payload;
 
 }
@@ -484,15 +487,18 @@ sub _build_bag_info {
     my ($self) = @_;
     my $bagit = $self->metadata_path;
     my $file = join("/", $bagit, "bag-info.txt");
-    open(my $BAGINFO, "<", $file) or die("Cannot read $file: $!");
-    my @lines;
-    foreach my $line (<$BAGINFO>) {
-        push @lines, $line;
+    if (-e $file) {
+        open(my $BAGINFO, "<", $file) or di("Cannot read $file: $!");
+        my @lines;
+        foreach my $line (<$BAGINFO>) {
+            push @lines, $line;
+        }
+        close($BAGINFO);
+        my $lines = join("", @lines);
+        return $self->_parse_bag_info($lines);
     }
-    close($BAGINFO);
-    my $lines = join("", @lines);
-    return $self->_parse_bag_info ($lines);
-
+    # bag-info.txt is optional
+    return;
 }
 
 sub _build_non_payload_files {
@@ -517,7 +523,7 @@ sub _build_non_payload_files {
     }
     #print "name: ".$File::Find::name."\n";
   }, $self->metadata_path);
-
+  @non_payload = sort @non_payload;
   return wantarray ? @non_payload : \@non_payload;
 
 }
@@ -558,6 +564,7 @@ sub __calc_digest {
     return $digest;
 }
 
+
 # calc digest
 # expects digestobj, expected_ref, array_ref of filenames
 # returns arrayref of hashes where each entry has
@@ -605,6 +612,34 @@ sub calc_digests {
     return \@digest_hashes;
 }
 
+sub check_if_payload_filepath_violates{
+    my $local_name = shift;
+    # HINT: there is no guarantuee *not* to escape!
+    return
+    ($local_name =~ m/^~/) # Unix Home
+        || ($local_name =~ m#\./#) # Unix, parent dir escape
+        || ($local_name =~ m#^[A-Z]:[\\/]#) # Windows Drive
+        || ($local_name =~ m#^/#) # Unix absolute path
+        || ($local_name =~ m#^$#) # Unix Env
+        || ($local_name =~ m#^\\#) # Windows absolute path
+        || ($local_name =~ m#^%[^%]*%#) # Windows ENV
+        || ($local_name =~ m#^\*#) # Artifact of md5sum-Tool, where ' *' is allowed to separate checksum and file in fixity line
+        || ($local_name =~ m#[<>:"?|]#) # Windows reserved chars
+        || ($local_name =~ m#(CON|PRN|AUX|NUL|COM[1-9]|LPT[1-9])#) # Windows reserved filenames
+    ;
+}
+
+sub normalize_payload_filepath {
+    my $filename = shift;
+    $filename =~ s#[\\](?![/])#/#g; # normalize Windows Backslashes, but only if they are no escape sequences
+    $filename =~ s#%#%25#g; # normalize percent
+    $filename =~ s#\x{0a}#%0A#g; #normalize NEWLINE
+    $filename =~ s#\x{0d}#%0D#g; #normlize CARRIAGE RETURN
+    $filename =~ s# #%20#g; # space
+    $filename =~ s#"##g; # quotes
+    return $filename;
+}
+
 sub _verify_XXX_manifests {
     my ($self, $xxprefix, $xxmanifest_entries, $files, $return_all_errors) =@_;
     # Read the manifest file
@@ -635,12 +670,12 @@ sub _verify_XXX_manifests {
         next unless (defined $manifest_alg); # FIXME_ errormessage?
         my $digestobj = $manifest_alg->algorithm();
         my $local_xxfilename = "${xxprefix}-${alg}.txt";
-        my $xxfilename = "${bagit}$local_xxfilename";
+        my $xxfilename = "${bagit}/$local_xxfilename";
 
         # first check if each file from payload exists in manifest_entries for given alg
         foreach my $local_name (@payload) {
             # local_name is relative to bagit base
-            unless (exists $xxmanifest_entries->{$alg}->{$local_name}) { # localname as value should exist!
+            unless (exists $xxmanifest_entries->{$alg}->{normalize_payload_filepath($local_name)}) { # localname as value should exist!
                 &$subref_invalid_report_or_die(
                     "file '$local_name' found, which is not in '$local_xxfilename' (bag-path:'$bagit')!",
                 );
@@ -649,15 +684,12 @@ sub _verify_XXX_manifests {
         # second check if each file from manifest_entries for given alg exists in payload
         foreach my $local_name (keys %{$xxmanifest_entries->{$alg}}) {
             if ( # to avoid escapes via manifest-files
-                ($local_name =~ m/^~/) ||
-                    ($local_name =~ m#\./#) ||
-                    ($local_name =~ m#^[A-Z]:[\\/]#) ||
-                    ($local_name =~ m#^/#)
+                check_if_payload_filepath_violates( $local_name )
             ) {
                 &$subref_invalid_report_or_die("file '$local_name' not allowed in '$local_xxfilename' (bag-path:'$bagit'")
             }
             else {
-                unless (List::Util::any {$_ eq $local_name} @payload) {
+                unless (List::Util::any {normalize_payload_filepath($_) eq $local_name} @payload) {
                     &$subref_invalid_report_or_die(
                         "file '$local_name' NOT found, but expected via '$local_xxfilename' (bag-path:'$bagit')!"
                     );
@@ -735,7 +767,6 @@ sub verify_bag {
     my $payload_dir   = $self->payload_path;
     my $return_all_errors = $opts->{return_all_errors};
 
-
     if (-f $fetch_file) {
         die("Fetching via file '$fetch_file' is not supported by current Archive::BagIt implementation")
     }
@@ -759,8 +790,6 @@ sub verify_bag {
 }
 
 
-
-
 sub calc_payload_oxum {
     my($self) = @_;
     my @payload = @{$self->payload_files};
@@ -785,6 +814,7 @@ sub calc_bagsize {
     else { return sprintf "%0.2f TB", $octets/(1024*1024*1024*1024); }
 }
 
+
 sub create_bagit {
     my($self) = @_;
     my $metadata_path = $self->metadata_path();
@@ -794,6 +824,7 @@ sub create_bagit {
     return 1;
 }
 
+
 sub create_baginfo {
     my($self) = @_; # because bag-info.txt allows multiple key-value-entries, hash is replaced
     $self->_add_or_replace_bag_info('Bagging-Date', POSIX::strftime("%F", gmtime(time)));
@@ -863,7 +894,6 @@ sub init_metadata {
 }
 
 
-
 sub make_bag {
   my ($class, $bag_path) = @_;
   my $isa = ref $class;
@@ -891,30 +921,39 @@ Archive::BagIt::Base
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 SYNOPSIS
 
 This modules will hopefully help with the basic commands needed to create
 and verify a bag. This part supports BagIt 1.0 according to RFC 8493 ([https://tools.ietf.org/html/rfc8493](https://tools.ietf.org/html/rfc8493)).
 
+You only need to know the following methods first:
+
+=head2 read a BagIt
+
     use Archive::BagIt::Base;
 
     #read in an existing bag:
     my $bag_dir = "/path/to/bag";
     my $bag = Archive::BagIt::Base->new($bag_dir);
 
+=head2 construct a BagIt around a payload
 
-    #construct bag in an existing directory
+    use Archive::BagIt::Base;
     my $bag2 = Archive::BagIt::Base->make_bag($bag_dir);
 
+=head2 verify a BagIt-dir
+
+    use Archive::BagIt::Base;
+
     # Validate a BagIt archive against its manifest
     my $bag3 = Archive::BagIt::Base->new($bag_dir);
-    my $is_valid = $bag3->verify_bag();
+    my $is_valid1 = $bag3->verify_bag();
 
     # Validate a BagIt archive against its manifest, report all errors
     my $bag4 = Archive::BagIt::Base->new($bag_dir);
-    my $is_valid = $bag4->verify_bag( {report_all_errors => 1} );
+    my $is_valid2 = $bag4->verify_bag( {report_all_errors => 1} );
 
 =head1 NAME
 
@@ -922,7 +961,7 @@ Archive::BagIt::Base
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 NAME
 
@@ -1005,6 +1044,8 @@ See L<https://datatracker.ietf.org/doc/rfc8493/?include_text=1> for details.
 
 =back
 
+=head1 METHODS
+
 =head2 Constructor
 
 The constructor sub, will create a bag with a single argument,
@@ -1043,11 +1084,18 @@ As default SHA512 and MD5 will be loaded and therefore used. If you want to crea
 checksum-algorithm, you could use this method to (re-)register it. It expects list of strings with namespace of type:
 Archive::BagIt::Plugin::Algorithm::XXX where XXX is your chosen fixity algorithm.
 
-=head2 verify_bag
+=head2 calc_digests($bagit, $digestobj, $filenames_ref, $opts)
 
-An interface to verify a bag.
+Method to calculate and return all digests for a a list of files using a Digest-object. This method will be overwritten by C<Archive::BagIt::Fast>.
 
-You might also want to check Archive::BagIt::Fast to see a more direct way of accessing files (and thus faster).
+If object ist build with option C<parallel>, the digests will be build in parallel.
+
+=head2 verify_bag($opts)
+
+A method to verify a bag deeply. If C<$opts> is set with C<{return_all_errors}> all fixity errors are reported.
+The default ist to die with error message if any error is detected.
+
+HINT: You might also want to check Archive::BagIt::Fast to see a more direct way of accessing files (and thus faster).
 
 =head2 calc_payload_oxum()
 
@@ -1057,17 +1105,25 @@ returns an array with octets and streamcount of payload-dir
 
 returns a string with human readable size of paylod
 
-=head2 store
+=head2 create_bagit()
 
-store a bagit-obj if bagit directory-structure was already constructed,
+creates a bagit.txt file
 
-=head2 init_metadata
+=head2 create_baginfo()
+
+creates a bag-info.tx
+
+=head2 store()
+
+store a bagit-obj if bagit directory-structure was already constructed.
+
+=head2 init_metadata()
 
 A constructor that will just create the metadata directory
 
 This won't make a bag, but it will create the conditions to do that eventually
 
-=head2 make_bag
+=head2 make_bag( $bag_path )
 
 A constructor that will make and return a bag from a directory,
 
