@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use parent "Archive::BagIt::Base";
 
-our $VERSION = '0.059'; # TRIAL VERSION
+our $VERSION = '0.060'; # TRIAL VERSION
 
 use IO::AIO;
 use Time::HiRes qw(time);
@@ -60,7 +60,7 @@ Archive::BagIt::Fast
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 NAME
 
@@ -68,12 +68,28 @@ Archive::BagIt::Fast
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 NAME
 
 Archive::BagIt::Fast - For people who are willing to rely on some other modules in order to get better performance
 
+=head1 HINTs
+
+Use this module only if you have *measured* that your environment has a benefit. The results vary highly depending on
+typical file size, filesystem and storage systems.
+
+=head1 METHODS
+
+=over
+
+=item calc_digests($bagit, $digestobj, $filenames_ref, $opts)
+
+Method to calculate and return all digests for a a list of files using a Digest-object. This method implements fast
+file access using memory mapped I/O by IO::AIO.
+
+=back
+
 =head1 AVAILABILITY
 
 The latest version of this module is available from the Comprehensive Perl
